from __future__ import division

import log
import eca

from sklearn import linear_model
import numpy as np

import random
import argparse

class Reservoir(object):
    def __init__(self, R, I, distractor_period, ca_size, ca_rule):
        self.I = I
        self.R = R
        self.distractor_period = distractor_period
        self.ca_size = ca_size
        self.ca_rule = ca_rule

        self.mappings = []
        for r in xrange(R):
            self.mappings.append(random.sample(xrange(ca_size), 4))

    def add_values(self, values):
        for r in xrange(self.R):
            for n in xrange(4):
                self.automatas[r][self.mappings[r][n]] = values[n]

    # Misleading name, also run every CA I iterations
    def get_feature_vector(self):
        output = []

        for a in self.automatas:
            for i in xrange(self.I):
                a.nextState()
                output.extend(a.getState())

        return output

    # Return a tuple with the CA output and the expected output for training
    def execute(self, input):
        # Actual output from CAs
        output = []

        """
        The expected output is 0,0,1 for the first five time steps, during the
        distractor period and the timestep before it should start replaying the
        pattern.
        """
        expected = [[0, 0, 1]] * (6 + self.distractor_period)
        for expected_out in input:
            # We only have 3 output channels
            expected.append(expected_out[0:3])

        self.automatas = []
        for r in xrange(R):
            self.automatas.append(eca.ECA(self.ca_size, self.ca_rule))

        for n in xrange(5):
            self.add_values(input[n])
            output.append(self.get_feature_vector())

        for n in xrange(self.distractor_period):
            self.add_values([0, 0, 1, 0])
            output.append(self.get_feature_vector())

        self.add_values([0, 0, 0, 1])
        output.append(self.get_feature_vector())

        for n in xrange(5):
            self.add_values([0, 0, 1, 0])
            output.append(self.get_feature_vector())

        return (output, expected)

logger = log.setup_logger()
parser = argparse.ArgumentParser(description = 'The 5-bit problem')
parser.add_argument('-r', default=8, type=int)
parser.add_argument('-i', default=8, type=int)
parser.add_argument('-t', default=200, type=int, help='Distractor period')
parser.add_argument('--ca-size', default=40, type=int)
parser.add_argument('--ca-rule', default=90, type=int)

args = parser.parse_args()

R = args.r
I = args.i
distractor_period = args.t
ca_size = args.ca_size
ca_rule = args.ca_rule

logger.info('Running 5 bit with paramteters R = %d, I = %d, T0=%d, CA size %d and CA Rule %d' % (R, I, distractor_period, ca_size, ca_rule))

node = Reservoir(R, I, distractor_period, ca_size, ca_rule)
regNode = linear_model.LinearRegression()

inputs = [
        [[0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0]],
        [[0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0]],
        [[0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0]],
        [[0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0]],
        [[0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0]],
        [[0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0]],
        [[0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0]],
        [[0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0]],
        [[0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0]],
        [[0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0]],
        [[0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0]],
        [[0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0]],
        [[0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0]],
        [[0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0]],
        [[0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0]],
        [[0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0]],
        [[1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0]],
        [[1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0]],
        [[1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0]],
        [[1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0]],
        [[1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0]],
        [[1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0]],
        [[1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0]],
        [[1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0]],
        [[1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0]],
        [[1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0]],
        [[1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0]],
        [[1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0]],
        [[1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0]],
        [[1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0], [1, 0, 0, 0]],
        [[1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [0, 1, 0, 0]],
        [[1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0], [1, 0, 0, 0]],
        ]

fit_X = []
fit_y = []

for n in xrange(32):
    x, y = node.execute(inputs[n])
    fit_X.extend(x)
    fit_y.extend(y)

regNode.fit(np.array(fit_X), np.array(fit_y))

passes = [True] * 32
i = 0
for n in xrange(32):
    i = n * 211
    for x in xrange(211):
        predicted = regNode.predict(np.array([fit_X[i]]))

        for y in xrange(3):
            if abs(predicted[0][y] - fit_y[i][y]) >= 0.5:
                passes[n] = False
                break

        i += 1

if all(passes):
    logger.info("Test passed, all output correctly classified")
else:
    logger.info("Test not passed, %d of %d patterns correctly classified" % (passes.count(True), len(passes)))
