import numpy as np
import random
import argparse
import Oger

from sklearn import linear_model

import log
import eca

def float_to_bits(float):
    bits = np.unpackbits(np.fromstring(float.tostring(), dtype=np.uint8))

    # The two most significant bits are always 0, so we don't include them in
    # the output. Not tested for cross-platform compatability
    return np.append(bits[0:56], bits[58:64])

class Reservoir(object):
    def __init__(self, R, I, input_size, ca_size, ca_rule, input_area):
        self.I = I
        self.R = R
        self.input_size = input_size
        self.ca_size = ca_size
        self.ca_rule = ca_rule

        assert input_area >= input_size

        self.mappings = []
        for r in xrange(R):
            mapping = random.sample(xrange(input_area), input_size)
            self.mappings.append(mapping)

    def add_values(self, values):
        for r in xrange(self.R):
            for key, value in enumerate(values):
                self.automatas[r][self.mappings[r][key]] = value

    def get_feature_vector(self):
        output = []

        for a in self.automatas:
            for i in xrange(self.I):
                a.nextState()
                output.extend(a.getState())

        return output

    def execute(self, input):
        output = []

        self.automatas = []
        for r in xrange(R):
            self.automatas.append(eca.ECA(self.ca_size, self.ca_rule))

        for sample in input:
            self.add_values(float_to_bits(sample[0]))
            output.append(self.get_feature_vector())

        return output

logger = log.setup_logger()
parser = argparse.ArgumentParser(description = 'Estimate a narma30 time series')
parser.add_argument('-r', default=16, type=int)
parser.add_argument('-i', default=16, type=int)
parser.add_argument('--ca-size', default=200, type=int)
parser.add_argument('--ca-rule', default=85, type=int)
parser.add_argument('--input-area', default=100, type=int)

args = parser.parse_args()

R = args.r
I = args.i
ca_size = args.ca_size
ca_rule = args.ca_rule
input_area = args.input_area

logger.info('Running narma30 with paramteters R = %d, I = %d, CA size %d, input area %d and CA Rule %d' % (R, I, ca_size, input_area, ca_rule))

reservoir = Reservoir(R, I, 62, ca_size, ca_rule, input_area)
estimator = linear_model.Ridge()

x, y = Oger.datasets.narma30(n_samples = 2)

estimator.fit(np.array(reservoir.execute(x[0])), y[0])

testout = estimator.predict(np.array(reservoir.execute(x[1])))
logger.info("NRMSE: %s" % (Oger.utils.nrmse(y[1], testout)))
