from __future__ import division

import log
import eca

from sklearn import linear_model
import numpy as np
import simplejson as json

import random
import argparse
import os

class Reservoir(object):
    def __init__(self, R, I, ca_size, ca_rule):
        assert(ca_size >= 747*29)
        self.I = I
        self.R = R
        self.ca_size = ca_size
        self.ca_rule = ca_rule

        self.mappings = []
        for r in xrange(R):
            self.mappings.append(random.sample(xrange(ca_size), 747 * 29))

    def add_values(self, values):
        for r in xrange(self.R):
            for (key, value) in enumerate(values):
                self.automatas[r][self.mappings[r][key]] = value

    def get_feature_vector(self):
        output = []

        for a in self.automatas:
            for i in xrange(self.I):
                a.nextState()
                output.extend(a.getState())

        return output

    def execute(self, input):
        output = []
        self.automatas = []

        for r in xrange(R):
            self.automatas.append(eca.ECA(self.ca_size, self.ca_rule))

        self.add_values(input)
        return self.get_feature_vector()

logger = log.setup_logger()
parser = argparse.ArgumentParser(description = 'Japanese vowel classification experiment')
parser.add_argument('-r', default=8, type=int)
parser.add_argument('-i', default=8, type=int)
parser.add_argument('--ca-size', default=21663, type=int)
parser.add_argument('--ca-rule', default=90, type=int)

args = parser.parse_args()

R = args.r
I = args.i
ca_size = args.ca_size
ca_rule = args.ca_rule

logger.info('Running japanese vowel classification with paramteters R = %d, I = %d, CA size %d and CA Rule %d' % (R, I, ca_size, ca_rule))

node = Reservoir(R, I, ca_size, ca_rule)
regNode = linear_model.SGDClassifier(loss="log", alpha=0.0000001)

with open(os.path.join(os.path.dirname(__file__), "vowels", "bitvalues.json")) as f:
    data = json.loads(f.read())

training_data = data["train"]
training_left = [len(a) for a in training_data]

while any(training_left):
    person = random.choice([a for a in enumerate(training_left) if a[1] > 0])[0]
    train = training_data[person].pop()
    training_left[person] -= 1

    out = np.array([node.execute(train)])
    regNode.partial_fit(out, np.array([person]), np.arange(9))

correct = 0
n_tests = 0
for person, tests in enumerate(data["test"]):
    for test in tests:
        n_tests += 1
        out = np.array([node.execute(test)])
        result = regNode.predict(out).tolist()[0]

        if result == person:
            correct += 1

logger.info("Test completed, %f%% correct classifications (%d/%d)" % ((correct / n_tests) * 100, correct, n_tests))
