import datetime
import logging
import sys
from termcolor import colored

class ColoredFormatter(logging.Formatter):
    def __init__(self, *args, **kwargs):
        super(ColoredFormatter, self).__init__(*args, **kwargs)

        self.level_colors = {
                logging.DEBUG: 'green',
                logging.INFO: 'grey',
                logging.WARNING: 'red',
                logging.ERROR: 'white',
                logging.CRITICAL: 'white',
                }

        self.level_backgrounds = {
                logging.DEBUG: 'grey',
                logging.INFO: 'green',
                logging.WARNING: 'yellow',
                logging.ERROR: 'red',
                logging.CRITICAL: 'red',
                }

        self.level_attrs = {
                logging.DEBUG: [],
                logging.INFO: [],
                logging.WARNING: [],
                logging.ERROR: ['bold'],
                logging.CRITICAL: ['bold', 'blink'],
                }

    def format(self, record):
        created = datetime.datetime.fromtimestamp(record.created)
        levelname = colored(
                "[%s]" % (record.levelname),
                self.level_colors[record.levelno],
                "on_" + self.level_backgrounds[record.levelno],
                attrs = self.level_attrs[record.levelno]
                )

        return "%s - %s - %s" % (created, levelname, record.msg)

def setup_logger(name = '', out = None, colors = True, level = logging.INFO):
    logger = logging.getLogger(name)
    logger.setLevel(level)

    if out:
        # We want to log to a file
        handler = logging.FileHandler(out)
    else:
        # We want to log to stderr
        handler = logging.StreamHandler(sys.stderr)

    if colors:
        handler.setFormatter(ColoredFormatter())
    else:
        handler.setFormatter(logging.Formatter('%(asctime)s [%(levelname)s] - %(message)s'))

    logger.addHandler(handler)

    return logger
